package com.sunyard.chsm.service;

import com.sunyard.chsm.auth.UserContext;
import com.sunyard.chsm.enums.*;
import com.sunyard.chsm.mapper.AppCertMapper;
import com.sunyard.chsm.mapper.KeyInfoMapper;
import com.sunyard.chsm.mapper.SpKeyRecordMapper;
import com.sunyard.chsm.model.entity.AppCert;
import com.sunyard.chsm.model.entity.KeyInfo;
import com.sunyard.chsm.model.entity.KeyRecord;
import com.sunyard.chsm.param.*;
import com.sunyard.chsm.sdf.SdfApiService;
import com.sunyard.chsm.sdf.context.AlgId;
import com.sunyard.chsm.sdf.model.EccCipher;
import com.sunyard.chsm.sdf.model.EccSignature;
import com.sunyard.chsm.sdf.util.LangUtils;
import com.sunyard.chsm.utils.CodecUtils;
import com.sunyard.chsm.utils.SignedAndEnvelopedData;
import com.sunyard.chsm.utils.gm.cert.BCSM2CertUtils;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.bouncycastle.asn1.*;
import org.bouncycastle.asn1.cms.*;
import org.bouncycastle.asn1.gm.GMObjectIdentifiers;
import org.bouncycastle.asn1.x500.X500Name;
import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
import org.bouncycastle.cert.X509CertificateHolder;
import org.bouncycastle.cert.jcajce.JcaCertStore;
import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
import org.bouncycastle.cert.jcajce.JcaX509CertificateHolder;
import org.bouncycastle.cms.*;
import org.bouncycastle.cms.jcajce.JcaSignerInfoGeneratorBuilder;
import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPublicKey;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.operator.*;
import org.bouncycastle.operator.jcajce.JcaDigestCalculatorProviderBuilder;
import org.bouncycastle.util.Store;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.security.NoSuchProviderException;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.Collections;
import java.util.Objects;

/**
 * @author liulu
 * @since 2024/12/20
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class AsymKeyService {

    private final AppCertMapper appCertMapper;
    private final KeyInfoMapper keyInfoMapper;
    private final SpKeyRecordMapper spKeyRecordMapper;
    private final SdfApiService sdfApiService;


    public AsymEncryptResp encrypt(AsymEncryptReq req) {
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        KeyInfo keyInfo = checkKey(req.getKeyId(), KeyUsage.ENCRYPT_DECRYPT);
        KeyAlg keyAlg = KeyAlg.of(keyInfo.getKeyAlg());
        Assert.notNull(keyAlg, "数据异常");
        KeyRecord keyRecord = spKeyRecordMapper.selectUsedByKeyId(keyInfo.getId());
        Assert.notNull(keyRecord, "数据异常");
        byte[] cipherData;
        switch (keyAlg) {
            case SM2:
                byte[] pub = CodecUtils.decodeHex(keyRecord.getPubKey());
                EccCipher eccCipher = sdfApiService.externalEncryptECC(pub, plainData);
                cipherData = eccCipher.getC1C3C2Bytes();
                break;
            default:
                throw new UnsupportedOperationException("不支持的密钥算法:" + keyAlg.getCode());
        }

        AsymEncryptResp resp = new AsymEncryptResp();
        resp.setKeyId(keyInfo.getId());
        resp.setKeyIndex(keyRecord.getKeyIndex());
        resp.setCipherData(CodecUtils.encodeBase64(cipherData));
        return resp;
    }

    public AsymDecryptResp decrypt(AsymDecryptReq req) {
        byte[] cipherData = CodecUtils.decodeBase64(req.getCipherData());
        KeyInfo keyInfo = checkKey(req.getKeyId(), KeyUsage.ENCRYPT_DECRYPT);
        KeyAlg keyAlg = KeyAlg.of(keyInfo.getKeyAlg());
        Assert.notNull(keyAlg, "数据异常");
        KeyRecord keyRecord = spKeyRecordMapper.selectById(Long.valueOf(req.getKeyIndex()));
        Assert.notNull(keyRecord, "数据异常");
        Assert.isTrue(Objects.equals(keyRecord.getKeyId(), keyInfo.getId()), "密钥Id和密钥索引不匹配");
        byte[] plainData;
        switch (keyAlg) {
            case SM2:
                byte[] encPri = CodecUtils.decodeHex(keyRecord.getKeyData());
                byte[] pri = sdfApiService.decryptByTMK(encPri);
                plainData = sdfApiService.externalDecryptECC(pri, cipherData);
                break;
            default:
                throw new UnsupportedOperationException("不支持的密钥算法:" + keyAlg.getCode());
        }
        AsymDecryptResp resp = new AsymDecryptResp();
        resp.setKeyId(keyInfo.getId());
        resp.setKeyIndex(keyRecord.getKeyIndex());
        resp.setPlainData(CodecUtils.encodeBase64(plainData));
        return resp;
    }

    public AsymSignRawResp signRaw(AsymSignRawReq req) {
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        byte[] userId = CodecUtils.decodeBase64(req.getUserId());
        KeyInfo keyInfo = checkKey(req.getKeyId(), KeyUsage.SIGN_VERIFY);
        KeyAlg keyAlg = KeyAlg.of(keyInfo.getKeyAlg());
        Assert.notNull(keyAlg, "数据异常");
        KeyRecord keyRecord = spKeyRecordMapper.selectUsedByKeyId(keyInfo.getId());
        Assert.notNull(keyRecord, "数据异常");
        byte[] signData;
        switch (keyAlg) {
            case SM2:
                byte[] encPri = CodecUtils.decodeHex(keyRecord.getKeyData());
                byte[] pri = sdfApiService.decryptByTMK(encPri);
                EccSignature signature = req.isPreProcess()
                        ? sdfApiService.externalSignWithIdECC(pri, plainData, userId)
                        : sdfApiService.externalSignECC(pri, plainData);
                signData = signature.getRawSignBytes();
                break;
            default:
                throw new UnsupportedOperationException("不支持的密钥算法:" + keyAlg.getCode());
        }

        AsymSignRawResp resp = new AsymSignRawResp();
        resp.setKeyId(keyInfo.getId());
        resp.setKeyIndex(keyRecord.getKeyIndex());
        resp.setSignData(CodecUtils.encodeBase64(signData));
        return resp;
    }


    public VerifyResp verifyRaw(AsymVerifyRawReq req) {
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        byte[] signData = CodecUtils.decodeBase64(req.getSignData());
        byte[] userId = CodecUtils.decodeBase64(req.getUserId());
        KeyInfo keyInfo = checkKey(req.getKeyId(), KeyUsage.SIGN_VERIFY);
        KeyAlg keyAlg = KeyAlg.of(keyInfo.getKeyAlg());
        Assert.notNull(keyAlg, "数据异常");
        KeyRecord keyRecord = spKeyRecordMapper.selectById(Long.valueOf(req.getKeyIndex()));
        Assert.notNull(keyRecord, "数据异常");
        Assert.isTrue(Objects.equals(keyRecord.getKeyId(), keyInfo.getId()), "密钥Id和密钥索引不匹配");
        boolean verified;
        switch (keyAlg) {
            case SM2:
                byte[] pub = CodecUtils.decodeHex(keyRecord.getPubKey());
                verified = req.isPreProcess()
                        ? sdfApiService.externalVerifyWithIdECC(pub, plainData, signData, userId)
                        : sdfApiService.externalVerifyECC(pub, plainData, signData);
                break;
            default:
                throw new UnsupportedOperationException("不支持的密钥算法:" + keyAlg.getCode());
        }
        VerifyResp resp = new VerifyResp();
        resp.setVerified(verified);
        return resp;
    }

    public AsymSignP7Resp signP7Attach(AsymSignP7Req req) {
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        AppCert appCert = appCertMapper.selectSignBySubject(req.getSubject());
        Assert.notNull(appCert, "证书主题对应的证书不存在");
        Assert.isTrue(Objects.equals(appCert.getApplicationId(), UserContext.getCurrentAppId()), "您无权使用此密钥ID");
        byte[] encPri = CodecUtils.decodeHex(appCert.getEncPriKey());
        byte[] pri = sdfApiService.decryptByTMK(encPri);

        byte[] asymSignP7Resp = p7Sign(pri, appCert.getCertText(), plainData, true);
        AsymSignP7Resp resp = new AsymSignP7Resp();
        resp.setSignData(CodecUtils.encodeBase64(asymSignP7Resp));
        return resp;
    }

    public VerifyResp verifyP7Attach(AsymVerifyP7Req req) {
        byte[] signData = CodecUtils.decodeBase64(req.getSignData());
        boolean verify;
        try {
            verify = p7Verify(signData, null);

        } catch (Exception e) {
            verify = false;
        }
        VerifyResp resp = new VerifyResp();
        resp.setVerified(verify);
        return resp;
    }

    public AsymSignP7Resp signP7Detach(AsymSignP7Req req) {
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        AppCert appCert = appCertMapper.selectSignBySubject(req.getSubject());
        Assert.notNull(appCert, "证书主题对应的证书不存在");
        Assert.isTrue(Objects.equals(appCert.getApplicationId(), UserContext.getCurrentAppId()), "您无权使用此证书");
        byte[] encPri = CodecUtils.decodeHex(appCert.getEncPriKey());
        byte[] pri = sdfApiService.decryptByTMK(encPri);

        byte[] asymSignP7Resp = p7Sign(pri, appCert.getCertText(), plainData, false);
        AsymSignP7Resp resp = new AsymSignP7Resp();
        resp.setSignData(CodecUtils.encodeBase64(asymSignP7Resp));
        return resp;
    }

    public VerifyResp verifyP7Detach(AsymVerifyP7Req req) {
        Assert.hasText(req.getPlainData(), "P7Detach验签原文不能为空");
        byte[] signData = CodecUtils.decodeBase64(req.getSignData());
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        boolean verify;
        try {
            verify = p7Verify(signData, plainData);

        } catch (Exception e) {
            verify = false;
        }
        VerifyResp resp = new VerifyResp();
        resp.setVerified(verify);
        return resp;
    }

    private byte[] p7Sign(byte[] pri, String cert, byte[] plainData, boolean encapsulate) {
        try {
            X509Certificate x509Cert = BCSM2CertUtils.getX509Cert(cert);
            // 构造签名内容
            CMSTypedData cmsData = new CMSProcessableByteArray(plainData);

            ContentSigner signer = new ContentSigner() {
                private final ByteArrayOutputStream stream = new ByteArrayOutputStream();

                @Override
                public AlgorithmIdentifier getAlgorithmIdentifier() {
                    return new AlgorithmIdentifier(GMObjectIdentifiers.sm2sign_with_sm3);
                }

                @Override
                public OutputStream getOutputStream() {
                    return stream;
                }

                @Override
                public byte[] getSignature() {
                    EccSignature signature = sdfApiService.externalSignWithIdECC(pri, stream.toByteArray(), null);
                    return signature.getDerSignBytes();
                }
            };

            // 生成签名者信息
            SignerInfoGenerator signerInfoGenerator = new JcaSignerInfoGeneratorBuilder(
                    new JcaDigestCalculatorProviderBuilder().setProvider(BouncyCastleProvider.PROVIDER_NAME).build()
            ).build(signer, x509Cert);
            // 构建 CMS Signed Data
            CMSSignedDataGenerator generator = new CMSSignedDataGenerator();
            generator.addSignerInfoGenerator(signerInfoGenerator);
            generator.addCertificates(new JcaCertStore(Collections.singletonList(x509Cert)));
            CMSSignedData signedData = generator.generate(cmsData, encapsulate);
            return signedData.getEncoded();
        } catch (Exception ex) {
            log.warn("", ex);
            throw new IllegalArgumentException("P7Attach 签名异常");
        }
    }

    public boolean p7Verify(byte[] signedDataBytes, byte[] originalData) throws Exception {

        CMSSignedData signedData;
        if (originalData == null || originalData.length == 0) {
            signedData = new CMSSignedData(signedDataBytes);
        } else {
            CMSTypedData originalContent = new CMSProcessableByteArray(originalData);
            signedData = new CMSSignedData(originalContent, signedDataBytes);
        }

        Store<X509CertificateHolder> certStore = signedData.getCertificates();
        SignerInformationStore signers = signedData.getSignerInfos();

        for (SignerInformation signer : signers.getSigners()) {
            Collection<X509CertificateHolder> matches = certStore.getMatches(signer.getSID());
            if (matches.isEmpty()) {
                throw new IllegalArgumentException("No matching certificate found for signer");
            }
            X509CertificateHolder certHolder = matches.iterator().next(); // 这里进行类型安全的提取
            X509Certificate cert = new JcaX509CertificateConverter()
                    .setProvider(BouncyCastleProvider.PROVIDER_NAME)
                    .getCertificate(certHolder);
            CMSSignatureAlgorithmNameGenerator sigAlgNameGen = new DefaultCMSSignatureAlgorithmNameGenerator();
            SignatureAlgorithmIdentifierFinder sigAlgIDFinder = new DefaultSignatureAlgorithmIdentifierFinder();
            DigestCalculatorProvider digestProvider = new JcaDigestCalculatorProviderBuilder().setProvider(BouncyCastleProvider.PROVIDER_NAME).build();
            SignerInformationVerifier verifier = new SignerInformationVerifier(sigAlgNameGen, sigAlgIDFinder, build(cert), digestProvider);

            if (signer.verify(verifier)) {
                return true;
            }
        }
        return false;
    }

    public ContentVerifierProvider build(X509Certificate certificate)
            throws OperatorCreationException {
        X509CertificateHolder certHolder;
        byte[] xy;
        try {
            certHolder = new JcaX509CertificateHolder(certificate);
            BCECPublicKey publicKey = (BCECPublicKey) certificate.getPublicKey();
            xy = LangUtils.merge(publicKey.getQ().getXCoord().getEncoded(), publicKey.getQ().getYCoord().getEncoded());
        } catch (CertificateEncodingException e) {
            throw new OperatorCreationException("cannot process certificate: " + e.getMessage(), e);
        }

        return new ContentVerifierProvider() {
            public boolean hasAssociatedCertificate() {
                return true;
            }

            public X509CertificateHolder getAssociatedCertificate() {
                return certHolder;
            }

            public ContentVerifier get(AlgorithmIdentifier algorithm) {
                return new ContentVerifier() {
                    private final ByteArrayOutputStream stream = new ByteArrayOutputStream();

                    @Override
                    public AlgorithmIdentifier getAlgorithmIdentifier() {
                        return algorithm;
                    }

                    @Override
                    public OutputStream getOutputStream() {
                        return stream;
                    }

                    @Override
                    public boolean verify(byte[] expected) {
                        return sdfApiService.externalVerifyWithIdECC(xy, stream.toByteArray(), expected, null);
                    }
                };
            }
        };
    }

    @SneakyThrows
    public AsymEnvelopeSealResp envelopeSeal(AsymEnvelopeSealReq req) {
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());

        EnvelopedData contentInfo = getEnvelopedData(req.getEncCert(), plainData);
        byte[] envelopeData = contentInfo.getEncoded("DER");

        AsymEnvelopeSealResp resp = new AsymEnvelopeSealResp();
        resp.setEnvelopeData(CodecUtils.encodeBase64(envelopeData));
        return resp;
    }

    private EnvelopedData getEnvelopedData(String encCert, byte[] plainData) throws CertificateException, NoSuchProviderException {
        X509Certificate cert = BCSM2CertUtils.getX509Cert(encCert);
        AlgorithmIdentifier symAlg = new AlgorithmIdentifier(GMObjectIdentifiers.sms4_ecb);
        AlgorithmIdentifier asymAlg = new AlgorithmIdentifier(GMObjectIdentifiers.sm2encrypt);

        byte[] symKey = sdfApiService.generateRandom(16);
        BCECPublicKey publicKey = (BCECPublicKey) cert.getPublicKey();
        byte[] xy = LangUtils.merge(publicKey.getQ().getXCoord().getEncoded(), publicKey.getQ().getYCoord().getEncoded());
        EccCipher cipher = sdfApiService.externalEncryptECC(xy, symKey);
        byte[] encryptKey = cipher.getC1C3C2Bytes();

        X500Name x500Name = X500Name.getInstance(cert.getIssuerX500Principal().getEncoded());
        KeyTransRecipientInfo keyTransRecipientInfo = new KeyTransRecipientInfo(
                new RecipientIdentifier(new IssuerAndSerialNumber(x500Name, cert.getSerialNumber())),
                asymAlg,
                new DEROctetString(encryptKey)
        );
        RecipientInfo recipientInfo = new RecipientInfo(keyTransRecipientInfo);

        byte[] encContent = sdfApiService.symEncrypt(AlgId.SGD_SM4_ECB, Padding.PCKS7Padding, symKey, null, plainData);
        EncryptedContentInfo encContentInfo = new EncryptedContentInfo(
                new ASN1ObjectIdentifier("1.2.156.10197.6.1.4.2.1"),
                symAlg,
                new DEROctetString(encContent)
        );

        return new EnvelopedData(null,
                new DERSet(recipientInfo),
                encContentInfo,
                (ASN1Set) null);
    }

    @SneakyThrows
    public AsymEnvelopeUnsealResp envelopeUnseal(AsymEnvelopeUnsealReq req) {
        byte[] envelopeData = CodecUtils.decodeBase64(req.getEnvelopeData());
        // 解密数字信封
        EnvelopedData ed = EnvelopedData.getInstance(envelopeData);
        byte[] plain = getPlainFromEnvelopedData(ed);
        AsymEnvelopeUnsealResp resp = new AsymEnvelopeUnsealResp();
        resp.setPlainData(CodecUtils.encodeBase64(plain));
        return resp;
    }

    private byte[] getPlainFromEnvelopedData(EnvelopedData ed) {
        ASN1Set infos = ed.getRecipientInfos();
        RecipientInfo recipientInfo = RecipientInfo.getInstance(infos.getObjectAt(0));
        KeyTransRecipientInfo transRecipientInfo = KeyTransRecipientInfo.getInstance(recipientInfo.getInfo());
        IssuerAndSerialNumber serialNumber = IssuerAndSerialNumber.getInstance(transRecipientInfo.getRecipientIdentifier().getId());
        String sn = serialNumber.getSerialNumber().getValue().toString();
        AppCert appCert = appCertMapper.selectBySN(sn);
        Assert.notNull(appCert, "此信封对应的证书不存在");
        Assert.isTrue(Objects.equals(appCert.getApplicationId(), UserContext.getCurrentAppId()), "您无权使用此信封的解密证书");

        AlgorithmIdentifier algorithm = transRecipientInfo.getKeyEncryptionAlgorithm();
        Assert.isTrue(Objects.equals(algorithm.getAlgorithm().getId(), GMObjectIdentifiers.sm2encrypt.getId()), "密钥加密算法只支持SM2");

        byte[] pri = sdfApiService.decryptByTMK(CodecUtils.decodeHex(appCert.getEncPriKey()));
        byte[] encryptedKey = transRecipientInfo.getEncryptedKey().getOctets();
        byte[] symKey = sdfApiService.externalDecryptECC(pri, encryptedKey);

        EncryptedContentInfo info = ed.getEncryptedContentInfo();
        Assert.isTrue(Objects.equals(info.getContentEncryptionAlgorithm().getAlgorithm().getId(), GMObjectIdentifiers.sms4_ecb.getId()), "内容加密算法只支持SM4_ECB");

        byte[] octets = info.getEncryptedContent().getOctets();
        byte[] plain = sdfApiService.symDecrypt(AlgId.SGD_SM4_ECB, Padding.PCKS7Padding, symKey, null, octets);
        return plain;
    }

    @SneakyThrows
    public AsymEnvelopeSealResp signedEnvelopeSeal(AsymEnvelopeSealReq req) {
        Assert.hasText(req.getSignSubject(), "签名证书主题不能为空");
        byte[] plainData = CodecUtils.decodeBase64(req.getPlainData());
        AppCert appCert = appCertMapper.selectSignBySubject(req.getSignSubject());
        Assert.notNull(appCert, "证书主题对应的签名证书不存在");
        Assert.isTrue(Objects.equals(appCert.getApplicationId(), UserContext.getCurrentAppId()), "您无权使用此证书");
        byte[] encPri = CodecUtils.decodeHex(appCert.getEncPriKey());
        byte[] pri = sdfApiService.decryptByTMK(encPri);

        byte[] asymSignP7Resp = p7Sign(pri, appCert.getCertText(), plainData, true);
        ContentInfo instance = ContentInfo.getInstance(asymSignP7Resp);
        SignedData signedData = SignedData.getInstance(instance.getContent());

        EnvelopedData envelopedData = getEnvelopedData(req.getEncCert(), plainData);

        SignedAndEnvelopedData signedAndEnvelopedData = new SignedAndEnvelopedData(
                new ASN1Integer(1),
                envelopedData.getRecipientInfos(),
                signedData.getDigestAlgorithms(),
                envelopedData.getEncryptedContentInfo(),
                signedData.getCertificates(),
                signedData.getCRLs(),
                signedData.getSignerInfos()
        );
        AsymEnvelopeSealResp resp = new AsymEnvelopeSealResp();
        resp.setEnvelopeData(CodecUtils.encodeBase64(signedAndEnvelopedData.getEncoded()));
        return resp;
    }

    @SneakyThrows
    public AsymEnvelopeUnsealResp signedEnvelopeUnseal(AsymEnvelopeUnsealReq req) {
        byte[] data = CodecUtils.decodeBase64(req.getEnvelopeData());
        SignedAndEnvelopedData signedAndEnvelopedData = SignedAndEnvelopedData.getInstance(data);

        EnvelopedData envelopedData = new EnvelopedData(
                null,
                signedAndEnvelopedData.getRecipientInfos(),
                signedAndEnvelopedData.getEncryptedContentInfo(),
                (ASN1Set) null
        );
        byte[] plainData = getPlainFromEnvelopedData(envelopedData);

        CMSTypedData cmsData = new CMSProcessableByteArray(plainData);
        ByteArrayOutputStream bOut = new ByteArrayOutputStream();
        cmsData.write(bOut);
        bOut.close();
        ContentInfo encInfo = new ContentInfo(cmsData.getContentType(), new BEROctetString(bOut.toByteArray()));
        SignedData sd = new SignedData(
                signedAndEnvelopedData.getDigestAlgorithms(),
                encInfo,
                signedAndEnvelopedData.getCertificates(),
                signedAndEnvelopedData.getCrls(),
                signedAndEnvelopedData.getSignerInfos()
        );
        ContentInfo contentInfo = new ContentInfo(CMSObjectIdentifiers.signedData, sd);
        p7Verify(contentInfo.getEncoded(), null);
        AsymEnvelopeUnsealResp resp = new AsymEnvelopeUnsealResp();
        resp.setPlainData(CodecUtils.encodeBase64(plainData));
        return resp;
    }


    private KeyInfo checkKey(Long keyId, KeyUsage usage) {

        KeyInfo keyInfo = keyInfoMapper.selectById(keyId);
        Assert.notNull(keyInfo, "密钥ID不存在");
        Assert.isTrue(Objects.equals(keyInfo.getApplicationId(), UserContext.getCurrentAppId()), "您无权使用此密钥ID");
        Assert.isTrue(KeyCategory.ASYM_KEY.getCode().equals(keyInfo.getKeyType()), "此密钥不是对称密钥");

        KeyStatus status = KeyStatus.of(keyInfo.getStatus());
        LocalDateTime now = LocalDateTime.now();
        Assert.isTrue(KeyStatus.ENABLED == status, "此密钥不是启用状态, 无法操作");
        Assert.isTrue(now.isAfter(keyInfo.getEffectiveTime()) && now.isBefore(keyInfo.getExpiredTime()), "此密钥不是启用状态, 无法操作");
        Assert.isTrue(KeyUsage.hasUsage(keyInfo.getKeyUsage(), usage), "此密钥无权进行" + usage.getDesc() + "操作");

        return keyInfo;
    }


}
